import { Core } from '@springtree/eva-services-core';
import { AnyAction } from 'redux';
import { IEvaServiceRequestAction } from './request.js';
import { IEvaServiceResponseAction } from './response.js';
import { IEvaServiceErrorAction } from './error.js';
/**
 * Current user action to set the current user token
 * Will trigger a current user data fetch
 *
 * @export
 * @interface IEvaCurrentUserTokenAction
 * @extends {AnyAction}
 */
export interface IEvaCurrentUserTokenAction extends AnyAction {
    type: 'CURRENT_USER_TOKEN';
    payload: {
        token?: string;
        serviceContext: string;
        chainContext: string;
        promise: Promise<string | undefined>;
        promiseChain: Promise<unknown[]>;
    };
}
/**
 * If a request with an Authorization token header present receives a 401
 * it will trigger this event. If this token is the current user token it
 * will trigger update logic
 *
 * @export
 * @interface IEvaUserTokenExpiredAction
 * @extends {AnyAction}
 */
export interface IEvaUserTokenExpiredAction extends AnyAction {
    type: 'USER_TOKEN_EXPIRED';
    payload: {
        token: string;
    };
}
/**
 * If a service response contains a (new) application token it will need to be
 * persisted to storage through the redux store.
 *
 * @export
 * @interface IEvaApplicationTokenAction
 * @extends {AnyAction}
 */
export interface IEvaApplicationTokenAction extends AnyAction {
    type: 'APPLICATION_TOKEN';
    payload: {
        token: string;
    };
}
/**
 * The combined type with all the actions for the current user
 */
export type TEvaCurrentUserActions = IEvaCurrentUserTokenAction | IEvaApplicationTokenAction | IEvaServiceRequestAction<Core.Login, 'login:REQUEST'> | IEvaServiceResponseAction<Core.Login, 'login:RESPONSE'> | IEvaServiceErrorAction<Core.Login, 'login:ERROR'> | IEvaServiceRequestAction<Core.Logout, 'logout:REQUEST'> | IEvaServiceResponseAction<Core.Logout, 'logout:RESPONSE'> | IEvaServiceErrorAction<Core.Logout, 'logout:ERROR'> | IEvaServiceRequestAction<Core.GetCurrentUser, 'currentUser:REQUEST'> | IEvaServiceResponseAction<Core.GetCurrentUser, 'currentUser:RESPONSE'> | IEvaServiceErrorAction<Core.GetCurrentUser, 'currentUser:ERROR'> | IEvaServiceRequestAction<Core.CreateCustomer, 'register:REQUEST'> | IEvaServiceResponseAction<Core.CreateCustomer, 'register:RESPONSE'> | IEvaServiceErrorAction<Core.CreateCustomer, 'register:ERROR'>;
/**
 * Helper method to update or unset the current user token
 *
 * @export
 * @param {string} [token='']
 * @returns
 */
export declare function createCurrentUserTokenAction(params?: {
    token?: string;
    chainContext?: string;
}): IEvaCurrentUserTokenAction;
/**
 * Helper method to create a login request action for the current user
 *
 * @export
 * @param {EVA.Core.Login} request
 * @returns
 */
export declare function createCurrentUserLoginAction(params: {
    data: Partial<EVA.Core.Login>;
    chainContext?: string;
}): IEvaServiceRequestAction<Core.Login, string>;
/**
 * Helper method to create a logout request action for the current user
 *
 * @export
 * @param {EVA.Core.Logout} request
 * @returns
 */
export declare function createCurrentUserLogoutAction(params?: {
    data?: EVA.Core.Logout;
    chainContext?: string;
}): IEvaServiceRequestAction<Core.Logout, string>;
/**
 * Helper method to create an update request action for the current user
 *
 * @export
 * @param {EVA.Core.GetCurrentUser} request
 * @returns
 */
export declare function createCurrentUserUpdateAction(params?: {
    data?: EVA.Core.GetCurrentUser;
    chainContext?: string;
}): IEvaServiceRequestAction<Core.GetCurrentUser, string>;
/**
 * Helper method to create a register user request action for the current user
 *
 * @export
 * @param {EVA.Core.CreateCustomer} request
 * @returns
 */
export declare function createCurrentUserRegisterAction(params: {
    request: EVA.Core.CreateCustomer;
    chainContext?: string;
}): IEvaServiceRequestAction<Core.CreateCustomer, string>;
//# sourceMappingURL=current-user.d.ts.map