import { IEvaServiceDefinition } from '@springtree/eva-services-core';
import { AnyAction } from 'redux';
import { IEvaServiceCallOptions } from '@springtree/eva-sdk-core-service';
import { IStateOptions } from './options.js';
/**
 * The data payload for an EVA Request Redux action
 *
 * @export
 * @interface IEvaServiceRequestAction
 * @extends {AnyAction}
 * @template SVC
 */
export interface IEvaServiceRequestActionPayload<SVC extends IEvaServiceDefinition> {
    data: SVC['request'];
    endpoint?: string;
    serviceOptions?: IEvaServiceCallOptions;
    serviceContext: string;
    chainContext: string;
    chainService: string;
    promise: Promise<SVC['response']>;
    promiseChain: Promise<unknown[]>;
}
export interface IEvaServiceRequestAction<SVC extends IEvaServiceDefinition, TYPE = string> extends AnyAction {
    type: TYPE;
    payload: IEvaServiceRequestActionPayload<SVC>;
}
/**
 * The parameters for the create request action helper
 *
 * @export
 * @interface ICreateServiceRequestActionParams
 * @template SVC
 */
export interface ICreateServiceRequestActionParams<SVC extends IEvaServiceDefinition> {
    /**
     * The EVA Service repository definition
     *
     */
    service: new () => SVC;
    /**
     * The EVA endpoint to use for the call.
     * Must be supplied if `endpointUrl` is not set in store state
     *
     * @type {EvaEndpoint}
     */
    endpoint?: string;
    /**
     * The typed EVA service request payload
     *
     * @type {SVC['request']}
     */
    payload?: SVC['request'];
    /**
     * The XHR call parameters to use
     *
     * @type {IEvaServiceCallOptions}
     */
    serviceOptions?: IEvaServiceCallOptions;
    /**
     * Optional configuration parameters to for the action and state engine
     *
     * @type {IStateOptions}
     */
    stateOptions?: IStateOptions;
    /**
     * Each service call has a unique context identifier.
     * Should be auto-generated but can also be supplied
     *
     * @type {string}
     */
    serviceContext?: string;
    /**
     * If a service request is part of a chain of calls this chainContext will be used
     * to identify the chain
     *
     * @type {string}
     */
    chainContext?: string;
    /**
     * If a service request is part of a chain of calls this chainService will contain
     * the name of the services the initiated the chain (if available)
     *
     * @type {string}
     */
    chainService?: string;
    /**
     * By default the action postfix will be REQUEST.
     * This property can be used to make other custom actions using the normal
     * context and promise setup.
     * The custom action not perform any function if no corresponding logic exists.
     *
     * @type {string}
     */
    customActionType?: string;
}
/**
 * Helper method to create a typed EVA Service request action
 *
 * @export
 * @template SVC
 * @param {ICreateServiceRequestAction<SVC>} params The request action parameters
 * @returns
 */
export declare function createServiceRequestAction<SVC extends IEvaServiceDefinition>(params: ICreateServiceRequestActionParams<SVC>): IEvaServiceRequestAction<SVC, string>;
//# sourceMappingURL=request.d.ts.map