import { IEvaServiceCallOptions } from '@springtree/eva-sdk-core-service';
import { Core, IEvaServiceDefinition } from '@springtree/eva-services-core';
import { AnyAction } from 'redux';
import { GetShoppingCartWithStatus, IGetShoppingCartWithStatusRequest } from '../helpers/get-shopping-cart-with-status.js';
import { IEvaServiceRequestActionPayload } from './request.js';
/**
 * Helper method to request a shopping cart update.
 * If no order id is specified in the request and no current order exists
 * this will not create a new order.
 * A new order will be created when the first product is added to the cart using
 * the createShoppingCartAddAction helper and logic.
 * This action could be used to resume the last known order on application
 * startup by providing an order id in the payload parameter
 *
 * @export
 * @param {{
 *     payload: IGetShoppingCartWithStatusRequest,
 *     serviceOptions?: IEvaServiceCallOptions;
 *   }} [params]
 * @returns
 */
export declare function createShoppingCartUpdateAction(params?: {
    payload: IGetShoppingCartWithStatusRequest;
    serviceOptions?: IEvaServiceCallOptions;
}): import("./request.js").IEvaServiceRequestAction<GetShoppingCartWithStatus, string>;
/**
 * Helper method to create a typed EVA ShoppingCart prepare for checkout action
 *
 * @export
 * @returns
 */
export declare function createShoppingCartPrepareForCheckoutAction(params?: {
    serviceOptions?: IEvaServiceCallOptions;
}): import("./request.js").IEvaServiceRequestAction<Core.PrepareOrderForCheckout, string>;
/**
 * These are the supported add to cart services
 */
export type TShoppingCartAddServices = Core.AddBundleProductToOrder | Core.AddDiscountCouponToOrder | Core.AddManualDiscountToOrder | Core.AddProductToOrder | Core.AddServiceProductToOrder | Core.AddUserBorrowedProductToOrder;
/**
 * The amended request action payload with the chosen service instance
 * and optional order type for when a new order needs to be created.
 *
 * @export
 * @interface IEvaShoppingCartAddActionPayload
 * @extends {IEvaServiceRequestActionPayload<SVC>}
 * @template SVC
 */
export interface IShoppingCartAddActionPayload<SVC extends IEvaServiceDefinition = TShoppingCartAddServices> extends IEvaServiceRequestActionPayload<SVC> {
    data: Omit<SVC['request'], 'OrderID' | 'SessionID' | 'TargetOrderID'>;
    service: new () => SVC;
    orderType?: EVA.Core.OrderTypes;
}
/**
 * The add to shopping cart action payload as it is to be sent through redux.
 * This is the format the logic will handle for the actual processing.
 *
 * @export
 * @interface IShoppingCartAddAction
 * @extends {AnyAction}
 */
export interface IShoppingCartAddAction extends AnyAction {
    type: 'shoppingCart:ADD';
    payload: IShoppingCartAddActionPayload;
}
/**
 * Helper method to create a typed EVA ShoppingCart add action
 *
 * @export
 * @param {IShoppingCartAddAction} params Details of which service to call and its specific payload
 * @returns
 */
export declare function createShoppingCartAddAction<SVC extends IEvaServiceDefinition = TShoppingCartAddServices>(params: {
    service: new () => SVC;
    payload: Omit<SVC['request'], 'OrderID' | 'SessionID' | 'TargetOrderID'>;
    orderType?: EVA.Core.OrderTypes;
    serviceOptions?: IEvaServiceCallOptions;
}): IShoppingCartAddAction;
/**
 * These are the supported modify to cart services
 */
export type TShoppingCartModifyingServices = Core.AttachCustomerToOrder | Core.CancelDiscountOrderLine | Core.CancelOrder | Core.ChangeOrderLinesToCarryOut | Core.ChangeOrderLinesToDelivery | Core.ChangeOrderLinesToPickup | Core.CreateCustomer | Core.CreateOrderLineUnitPriceCorrection | Core.DetachCustomerFromOrder | Core.Login | Core.ModifyOrderLinePrice | Core.ModifyQuantityOrdered | Core.PrepareOrderForCheckout | Core.RemoveOrderLineUnitPriceCorrection | Core.SetGiftWrappingOptionsOnOrder | Core.SetPickProductDiscountOptionsForOrderLine | Core.SetRequestedDate | Core.SplitOrderLine | Core.UpdateOrderBillingAddress | Core.UpdateOrderShippingAddress | Core.UpdateProductRequirementValuesForOrderLine | Core.UpdateSerialNumber;
/**
 * The amended request action payload with the chosen service instance
 *
 * @export
 * @interface IEvaShoppingCartModifyActionPayload
 * @extends {IEvaServiceRequestActionPayload<SVC>}
 * @template SVC
 */
export interface IShoppingCartModifyActionPayload<SVC extends IEvaServiceDefinition = TShoppingCartModifyingServices> extends IEvaServiceRequestActionPayload<SVC> {
    data: Omit<SVC['request'], 'OrderID' | 'SessionID' | 'TargetOrderID'>;
    service: new () => SVC;
    orderType?: EVA.Core.OrderTypes;
}
/**
 * The modify shopping cart action payload as it is to be sent through redux.
 * This is the format the logic will handle for the actual processing.
 *
 * @export
 * @interface IShoppingCartModifyAction
 * @extends {AnyAction}
 */
export interface IShoppingCartModifyAction extends AnyAction {
    type: 'shoppingCart:MODIFY';
    payload: IShoppingCartModifyActionPayload;
}
/**
 * Helper method to create a typed EVA ShoppingCart modify action
 *
 * @export
 * @param {IShoppingCartModifyAction} params Details of which service to call and its specific payload
 * @returns
 */
export declare function createShoppingCartModifyAction<SVC extends IEvaServiceDefinition = TShoppingCartModifyingServices>(params: {
    service: new () => SVC;
    payload: Omit<SVC['request'], 'OrderID' | 'SessionID' | 'TargetOrderID'>;
    serviceOptions?: IEvaServiceCallOptions;
}): IShoppingCartModifyAction;
//# sourceMappingURL=shopping-cart.d.ts.map