import { Core } from '@springtree/eva-services-core';
import { GetShoppingCartWithStatus } from '../helpers/get-shopping-cart-with-status.js';
import { createServiceRequestAction } from './request.js';
/**
 * Helper method to request a shopping cart update.
 * If no order id is specified in the request and no current order exists
 * this will not create a new order.
 * A new order will be created when the first product is added to the cart using
 * the createShoppingCartAddAction helper and logic.
 * This action could be used to resume the last known order on application
 * startup by providing an order id in the payload parameter
 *
 * @export
 * @param {{
 *     payload: IGetShoppingCartWithStatusRequest,
 *     serviceOptions?: IEvaServiceCallOptions;
 *   }} [params]
 * @returns
 */
export function createShoppingCartUpdateAction(params) {
    const action = createServiceRequestAction({
        service: GetShoppingCartWithStatus,
        stateOptions: {
            stateKey: 'shoppingCart',
        },
        payload: params?.payload,
        customActionType: 'UPDATE',
        serviceOptions: params?.serviceOptions,
    });
    return action;
}
/**
 * Helper method to create a typed EVA ShoppingCart prepare for checkout action
 *
 * @export
 * @returns
 */
export function createShoppingCartPrepareForCheckoutAction(params) {
    const action = createServiceRequestAction({
        service: Core.PrepareOrderForCheckout,
        stateOptions: {
            stateKey: 'shoppingCart',
        },
        customActionType: 'PREPARE_CHECKOUT',
        serviceOptions: params?.serviceOptions,
    });
    return action;
}
/**
 * Helper method to create a typed EVA ShoppingCart add action
 *
 * @export
 * @param {IShoppingCartAddAction} params Details of which service to call and its specific payload
 * @returns
 */
export function createShoppingCartAddAction(params) {
    const serviceAction = createServiceRequestAction({
        service: params.service,
        payload: params.payload,
        stateOptions: {
            stateKey: 'shoppingCart',
        },
        customActionType: 'ADD',
        serviceOptions: params.serviceOptions,
    });
    const cartAction = {
        type: 'shoppingCart:ADD',
        payload: {
            ...serviceAction.payload,
            service: params.service,
            chainService: serviceAction.payload.chainService,
            orderType: params.orderType,
        },
    };
    return cartAction;
}
/**
 * Helper method to create a typed EVA ShoppingCart modify action
 *
 * @export
 * @param {IShoppingCartModifyAction} params Details of which service to call and its specific payload
 * @returns
 */
export function createShoppingCartModifyAction(params) {
    const serviceAction = createServiceRequestAction({
        service: params.service,
        payload: params.payload,
        stateOptions: {
            stateKey: 'shoppingCart',
        },
        customActionType: 'MODIFY',
        serviceOptions: params.serviceOptions,
    });
    const cartAction = {
        type: 'shoppingCart:MODIFY',
        payload: {
            ...serviceAction.payload,
            service: params.service,
            chainService: serviceAction.payload.chainService,
        },
    };
    return cartAction;
}
//# sourceMappingURL=shopping-cart.js.map