import { logger } from '@springtree/eva-sdk-core-logger';
import { bootstrapEndpoint } from '@springtree/eva-sdk-core-service';
import { Core } from '@springtree/eva-services-core';
import { createLogic } from 'redux-logic';
import { getServicePromise } from '../promise.js';
import { getStoreSettings } from '../settings.js';
// We need to recurse the child product arrays to find all stock holding products
//
const getStockHoldingProductIds = (products) => {
    // Add the product id itself and recursively add the child item ids
    //
    const productIds = [];
    products.forEach((product) => {
        // Stock holding means not being configurable themselves
        if (product.Type !== 16384) {
            productIds.push(product.ProductID);
        }
        const childProductIds = getStockHoldingProductIds(product.Children || []);
        productIds.push(...childProductIds);
    });
    return productIds;
};
/**
 * The composite service definition for the ExtendedProductDetails helper
 *
 * @export
 * @class ExtendedProductDetails
 * @implements {IEvaServiceDefinition}
 */
export class ExtendedProductDetails {
    name = 'Composite:ExtendedProductDetails';
    path = '/api/composite/ExtendedProductDetails';
    request;
    response;
}
/**
 * Composite service request logic builder for the ExtendedProductDetails helper
 *
 * @export
 * @template SHAPE
 * @param {IExtendedProductDetailsLogicOptions} [stateOptions]
 * @returns
 */
export function createExtendedProductDetailsLogic(stateOptions) {
    // Determine the state key name either by the service name or the provided
    // custom state key name
    //
    const serviceName = (stateOptions && stateOptions.stateKey) || new ExtendedProductDetails().name;
    const requestLogic = createLogic({
        type: `${serviceName}:REQUEST`,
        processOptions: {
            dispatchReturn: true,
            successType: `${serviceName}:RESPONSE`,
            failType: `${serviceName}:ERROR`,
        },
        async process({ getState, action }) {
            const storeState = getState();
            const settings = getStoreSettings();
            const payload = action.payload;
            const endpointUrl = payload.endpoint ?? storeState.endpointUrl;
            const servicePromise = getServicePromise(payload.serviceContext);
            // Ensure we have an endpoint to use
            //
            if (!endpointUrl) {
                const failure = new Error('No EVA endpoint provided for service call');
                servicePromise.reject(failure);
                throw failure;
            }
            // Ensure we have a product identifier
            //
            if (!payload.data?.details?.ID) {
                console.warn('Missing product ID for request');
                servicePromise.resolve();
                return;
            }
            // Bootstrapping the store to get the EvaEndpoint instance we need
            // The SDK caches bootstrap results so after the first call this will be instant
            //
            const endpoint = await bootstrapEndpoint({ uri: endpointUrl });
            // Set provided chain context header
            //
            const serviceNameForProductDetail = new Core.GetProductDetail().name;
            const serviceOptions = Object.assign({
                chainContext: payload.chainContext,
            }, payload.serviceOptions);
            // Set the current user authentication token if needed
            //
            const currentUserToken = storeState.currentUserToken;
            if (!serviceOptions.authenticationToken && currentUserToken) {
                serviceOptions.authenticationToken = currentUserToken;
            }
            // Set the current application token if needed
            //
            const currentApplicationToken = storeState.currentApplicationToken;
            if (!serviceOptions.applicationToken && currentApplicationToken) {
                serviceOptions.applicationToken = currentApplicationToken;
            }
            // Call the product details service and return the response so the reducers can
            // write it to the store state
            //
            logger.debug(`[EVA:SDK:LOGIC:COMPOSITE:${serviceName}] Call service ${serviceNameForProductDetail}`, payload, serviceOptions)();
            try {
                const currentProduct = await endpoint.callService(Core.GetProductDetail, payload.data.details, serviceOptions);
                // Determine the product based on its type
                //
                const productId = payload.data.details.ID;
                const productTypes = currentProduct?.Result?.product_types || [];
                const isBundle = productTypes.indexOf(2048) !== -1;
                const isConfigurable = productTypes.indexOf(16384) !== -1;
                // Prepare the array that will contain all the involved product ids
                // These will be used to make single request for availability, prices, etc
                //
                const allProductIds = [productId];
                // Prepare the pickup organization unit(s) based on the current users
                // current organization
                //
                const organizationUnitsForPickup = [];
                const currentUser = storeState.currentUser?.data.response;
                if (currentUser?.User?.CurrentOrganizationID) {
                    organizationUnitsForPickup.push(currentUser.User.CurrentOrganizationID);
                }
                // Prepare response so we can fill in all the optional type and availability
                //
                const responsePayload = {
                    serviceContext: payload.serviceContext,
                    chainContext: payload.chainContext,
                    data: {
                        details: currentProduct,
                    },
                };
                if (isBundle) {
                    const requestBundleDetails = {
                        IncludedFields: settings.defaultProductProperties,
                        ...payload.data?.bundleDetails,
                        BundleProductID: productId,
                    };
                    const serviceNameForBundleProductDetails = new Core.GetBundleProductDetails().name;
                    const serviceOptionsForBundleProductDetails = Object.assign({
                        chainContext: payload.chainContext,
                    }, serviceOptions);
                    logger.debug(`[EVA:SDK:LOGIC:COMPOSITE:${serviceName}] Call service ${serviceNameForBundleProductDetails}`, requestBundleDetails, serviceOptions)();
                    responsePayload.data.bundleDetails = await endpoint.callService(Core.GetBundleProductDetails, requestBundleDetails, serviceOptionsForBundleProductDetails);
                    const bundleProducts = responsePayload.data?.bundleDetails?.Products ?? [];
                    bundleProducts.forEach((bundleProduct) => {
                        // Add the product ID for the product availability and prices request
                        //
                        allProductIds.push(bundleProduct.ID);
                    });
                }
                if (isConfigurable) {
                    const requestConfigurableDetails = {
                        IncludedFields: settings.defaultProductProperties,
                        ...payload.data?.configurableDetails,
                        ID: productId,
                    };
                    const serviceNameForConfigurableProductDetails = new Core.GetConfigurableProductDetail().name;
                    const serviceOptionsForConfigurableProductDetails = Object.assign({
                        chainContext: payload.chainContext,
                    }, serviceOptions);
                    logger.debug(`[EVA:SDK:LOGIC:COMPOSITE:${serviceName}] Call service ${serviceNameForConfigurableProductDetails}`, requestConfigurableDetails, serviceOptions)();
                    const configurableDetails = await endpoint.callService(Core.GetConfigurableProductDetail, requestConfigurableDetails, serviceOptionsForConfigurableProductDetails);
                    responsePayload.data.configurableDetails = configurableDetails;
                    // We will fetch availability for all stock holding products that are
                    // an option for this configurable product
                    //
                    allProductIds.push(...getStockHoldingProductIds(configurableDetails.Configurable.Children));
                }
                if (stateOptions?.disableAvailability !== true) {
                    const requestAvailabilityPayload = {
                        Options: {
                            Bundles: {
                                IncludeBundleInformation: true,
                            },
                            Delivery: {
                                AvailabilityDate: true,
                            },
                            Pickup: {
                                AvailabilityDate: true,
                                OrganizationUnitIDs: organizationUnitsForPickup,
                            },
                            QuantityAvailable: true,
                            ProductAvailabilityDetails: {
                                OrganizationUnitIDs: organizationUnitsForPickup,
                                ProductIDs: allProductIds,
                            },
                        },
                        ...payload.data?.availability,
                        // Override the product ids
                        //
                        Products: allProductIds.map((productId) => { return { ID: productId, QuantityRequested: 1 }; }),
                    };
                    const serviceNameForProductAvailability = new Core.GetProductAvailability().name;
                    const serviceOptionsForProductAvailability = Object.assign({
                        chainContext: payload.chainContext,
                    }, serviceOptions);
                    logger.debug(`[EVA:SDK:LOGIC:COMPOSITE:${serviceName}] Call service ${serviceNameForProductAvailability}`, requestAvailabilityPayload, serviceOptions)();
                    try {
                        responsePayload.data.availability = await endpoint.callService(Core.GetProductAvailability, requestAvailabilityPayload, serviceOptionsForProductAvailability);
                    }
                    catch (error) {
                        logger.warn(`[EVA:SDK:LOGIC:COMPOSITE:${serviceName}] Call service ${serviceNameForProductAvailability} failed`, error)();
                    }
                }
                if (stateOptions?.disableProductPrices !== true) {
                    const requestProductPricesPayload = {
                        ProductIDs: allProductIds,
                    };
                    const serviceNameForProductPrices = new Core.GetProductPrices().name;
                    const serviceOptionsForProductPrices = Object.assign({
                        chainContext: payload.chainContext,
                    }, serviceOptions);
                    logger.debug(`[EVA:SDK:LOGIC:COMPOSITE:${serviceName}] Call service ${serviceNameForProductPrices}`, requestProductPricesPayload, serviceOptions)();
                    try {
                        responsePayload.data.prices = await endpoint.callService(Core.GetProductPrices, requestProductPricesPayload, serviceOptionsForProductPrices);
                    }
                    catch (error) {
                        logger.warn(`[EVA:SDK:LOGIC:COMPOSITE:${serviceName}] Call service ${serviceNameForProductPrices} failed`, error)();
                    }
                }
                return responsePayload;
            }
            catch (error) {
                logger.error(`[EVA:SDK:LOGIC:COMPOSITE:${serviceName}] Service error`, error)();
                const errorPayload = {
                    serviceContext: payload.serviceContext,
                    chainContext: payload.chainContext,
                    data: error,
                };
                // If this an HTTPError from fetch we can try to extract the JSON response
                //
                try {
                    const httpError = error;
                    const jsonError = await httpError.response?.json();
                    errorPayload.data = jsonError || httpError.message;
                    errorPayload.httpResponse = httpError.response?.status;
                    const headers = {};
                    httpError.response?.headers.forEach((value, key) => (headers[key.toLowerCase()] = value));
                    errorPayload.headers = headers;
                }
                catch (jsonParseError) {
                    logger.warn(`[EVA:SDK:LOGIC:${serviceName}] Failed to parse JSON error response`, jsonParseError)();
                }
                servicePromise.reject(errorPayload);
                throw errorPayload;
            }
        },
    });
    return [
        requestLogic,
    ];
}
//# sourceMappingURL=extended-product-details.js.map