import { logger } from '@springtree/eva-sdk-core-logger';
import { bootstrapEndpoint } from '@springtree/eva-sdk-core-service';
import { Core } from '@springtree/eva-services-core';
import { createLogic } from 'redux-logic';
import { getServicePromise } from '../promise.js';
import { getStoreSettings } from '../settings.js';
/**
 * The composite service definition for the ExtendedSearchProducts helper
 *
 * @export
 * @class ExtendedSearchProducts
 * @implements {IEvaServiceDefinition}
 */
export class ExtendedSearchProducts {
    name = 'Composite:ExtendedSearchProducts';
    path = '/api/composite/ExtendedSearchProducts';
    request;
    response;
}
/**
 * Composite service request logic builder for the ExtendedSearchProducts helper
 *
 * @export
 * @template SHAPE
 * @param {IExtendedSearchProductsLogicOptions} [stateOptions]
 * @returns
 */
export function createExtendedSearchProductsLogic(stateOptions) {
    // Determine the state key name either by the service name or the provided
    // custom state key name
    //
    const serviceName = (stateOptions && stateOptions.stateKey) || new ExtendedSearchProducts().name;
    const requestLogic = createLogic({
        type: `${serviceName}:REQUEST`,
        processOptions: {
            dispatchReturn: true,
            successType: `${serviceName}:RESPONSE`,
            failType: `${serviceName}:ERROR`,
        },
        async process({ getState, action }) {
            const storeState = getState();
            const settings = getStoreSettings();
            const payload = action.payload;
            const endpointUrl = payload.endpoint || storeState.endpointUrl;
            const servicePromise = getServicePromise(payload.serviceContext);
            // Ensure we have an endpoint to use
            //
            if (!endpointUrl) {
                const failure = new Error('No EVA endpoint provided for service call');
                servicePromise.reject(failure);
                throw failure;
            }
            if (!payload.data) {
                payload.data = {};
            }
            // Bootstrapping the store to get the EvaEndpoint instance we need
            // The SDK caches bootstrap results so after the first call this will be instant
            //
            const endpoint = await bootstrapEndpoint({ uri: endpointUrl });
            // Set provided chain context header
            //
            const serviceNameForSearchProducts = new Core.SearchProducts().name;
            const serviceOptions = Object.assign({
                chainContext: payload.chainContext,
            }, payload.serviceOptions);
            // Set the current user authentication token if needed
            //
            const currentUserToken = storeState.currentUserToken;
            if (!serviceOptions.authenticationToken && currentUserToken) {
                serviceOptions.authenticationToken = currentUserToken;
            }
            // Set the current application token if needed
            //
            const currentApplicationToken = storeState.currentApplicationToken;
            if (!serviceOptions.applicationToken && currentApplicationToken) {
                serviceOptions.applicationToken = currentApplicationToken;
            }
            // Call the service and return the response so the reducers can
            // write it to the store state
            //
            logger.debug(`[EVA:SDK:LOGIC:COMPOSITE:${serviceName}] Call service ${serviceNameForSearchProducts}`, payload, serviceOptions)();
            try {
                const searchResult = await endpoint.callService(Core.SearchProducts, {
                    IncludedFields: settings.defaultProductProperties,
                    ...payload.data.search,
                }, serviceOptions);
                // Extract the product identifiers from the response and fetch their availability and prices
                //
                let availability = undefined;
                let prices = undefined;
                const searchProducts = searchResult?.Products ?? [];
                const productIds = searchProducts.map(product => product?.product_id).filter(productId => !!productId);
                if (productIds.length) {
                    const organizationUnitsForPickup = [];
                    const currentUser = storeState.currentUser?.data.response;
                    if (currentUser?.User?.CurrentOrganizationID) {
                        organizationUnitsForPickup.push(currentUser.User.CurrentOrganizationID);
                    }
                    const requestPayload = {
                        Options: {
                            Bundles: {
                                IncludeBundleInformation: true,
                            },
                            Delivery: {
                                AvailabilityDate: true,
                            },
                            Pickup: {
                                AvailabilityDate: true,
                                OrganizationUnitIDs: organizationUnitsForPickup,
                            },
                            QuantityAvailable: true,
                            ProductAvailabilityDetails: {
                                OrganizationUnitIDs: organizationUnitsForPickup,
                                ProductIDs: productIds,
                            },
                        },
                        ...payload.data.availability,
                        // Override the product ids
                        //
                        Products: productIds.map((productId) => { return { ID: productId, QuantityRequested: 1 }; }),
                    };
                    // Set provided chain context header
                    //
                    const serviceNameForProductAvailability = new Core.GetProductAvailability().name;
                    const serviceOptionsForProductAvailability = Object.assign({
                        chainContext: payload.chainContext,
                    }, serviceOptions);
                    logger.debug(`[EVA:SDK:LOGIC:COMPOSITE:${serviceName}] Call service ${serviceNameForProductAvailability}`, requestPayload, serviceOptions)();
                    try {
                        availability = await endpoint.callService(Core.GetProductAvailability, requestPayload, serviceOptionsForProductAvailability);
                    }
                    catch (error) {
                        logger.warn(`[EVA:SDK:LOGIC:COMPOSITE:${serviceName}] Call service ${serviceNameForProductAvailability} failed`, error)();
                    }
                    if (!stateOptions?.disableProductPrices) {
                        const requestPayload = {
                            ProductIDs: productIds,
                        };
                        // Set provided chain context header
                        //
                        const serviceNameForProductPrices = new Core.GetProductPrices().name;
                        const serviceOptionsForProductPrices = Object.assign({
                            chainContext: payload.chainContext,
                        }, serviceOptions);
                        logger.debug(`[EVA:SDK:LOGIC:COMPOSITE:${serviceName}] Call service ${serviceNameForProductPrices}`, requestPayload, serviceOptions)();
                        try {
                            prices = await endpoint.callService(Core.GetProductPrices, requestPayload, serviceOptionsForProductPrices);
                        }
                        catch (error) {
                            logger.warn(`[EVA:SDK:LOGIC:COMPOSITE:${serviceName}] Call service ${serviceNameForProductPrices} failed`, error)();
                        }
                    }
                }
                const responsePayload = {
                    serviceContext: payload.serviceContext,
                    chainContext: payload.chainContext,
                    data: {
                        searchResult,
                        availability,
                        prices,
                    },
                };
                return responsePayload;
            }
            catch (error) {
                logger.error(`[EVA:SDK:LOGIC:COMPOSITE:${serviceName}] Service error`, error)();
                const errorPayload = {
                    serviceContext: payload.serviceContext,
                    chainContext: payload.chainContext,
                    data: error,
                };
                // If this an HTTPError from fetch we can try to extract the JSON response
                //
                try {
                    const httpError = error;
                    const jsonError = await httpError.response?.json();
                    errorPayload.data = jsonError || httpError.message;
                    errorPayload.httpResponse = httpError.response?.status;
                    const headers = {};
                    httpError.response?.headers.forEach((value, key) => (headers[key.toLowerCase()] = value));
                    errorPayload.headers = headers;
                }
                catch (jsonParseError) {
                    logger.warn(`[EVA:SDK:LOGIC:${serviceName}] Failed to parse JSON error response`, jsonParseError)();
                }
                servicePromise.reject(errorPayload);
                throw errorPayload;
            }
        },
    });
    return [
        requestLogic,
    ];
}
//# sourceMappingURL=extended-search-products.js.map