import { logger } from '@springtree/eva-sdk-core-logger';
import { bootstrapEndpoint } from '@springtree/eva-sdk-core-service';
import { Core } from '@springtree/eva-services-core';
import { createLogic } from 'redux-logic';
import { getServicePromise } from '../promise.js';
import { getStoreSettings } from '../settings.js';
import { determineOrderStatus } from './order-status.js';
import { countOrderProductsByType } from './product-count-by-type.js';
/**
 * The composite service definition for the GetShoppingCartWithStatus helper
 *
 * @export
 * @class GetShoppingCartWithStatus
 * @implements {IEvaServiceDefinition}
 */
export class GetShoppingCartWithStatus {
    name = 'Composite:GetShoppingCartWithStatus';
    path = '/api/composite/GetShoppingCartWithStatus';
    request;
    response;
}
/**
 * Composite service request logic builder for the GetShoppingCartWithStatus helper
 *
 * @export
 * @template SHAPE
 * @param {IGetShoppingCartWithStatusLogicOptions} [stateOptions]
 * @returns
 */
export function createGetShoppingCartWithStatusLogic(stateOptions) {
    // Determine the state key name either by the service name or the provided
    // custom state key name
    //
    const serviceName = (stateOptions && stateOptions.stateKey) || new GetShoppingCartWithStatus().name;
    const requestLogic = createLogic({
        type: `${serviceName}:REQUEST`,
        processOptions: {
            dispatchReturn: true,
            successType: `${serviceName}:RESPONSE`,
            failType: `${serviceName}:ERROR`,
        },
        async process({ getState, action }) {
            const storeState = getState();
            const settings = getStoreSettings();
            const payload = action.payload;
            const endpointUrl = payload.endpoint || storeState.endpointUrl;
            const servicePromise = getServicePromise(payload.serviceContext);
            // Ensure we have an endpoint to use
            //
            if (!endpointUrl) {
                const failure = new Error('No EVA endpoint provided for service call');
                servicePromise.reject(failure);
                throw failure;
            }
            // Ensure we have an order identifier
            //
            if (!payload.data?.order?.OrderID) {
                console.warn('Missing order ID for request');
                servicePromise.resolve();
                return;
            }
            // Bootstrapping the store to get the EvaEndpoint instance we need
            // The SDK caches bootstrap results so after the first call this will be instant
            //
            const endpoint = await bootstrapEndpoint({ uri: endpointUrl });
            // Set provided chain context header
            //
            const serviceNameForGetShoppingCart = new Core.GetShoppingCart().name;
            const serviceOptions = Object.assign({
                chainContext: payload.chainContext,
            }, payload.serviceOptions);
            // Set the current user authentication token if needed
            //
            const currentUserToken = storeState.currentUserToken;
            if (!serviceOptions.authenticationToken && currentUserToken) {
                serviceOptions.authenticationToken = currentUserToken;
            }
            // Set the current application token if needed
            //
            const currentApplicationToken = storeState.currentApplicationToken;
            if (!serviceOptions.applicationToken && currentApplicationToken) {
                serviceOptions.applicationToken = currentApplicationToken;
            }
            // Call the product details service and return the response so the reducers can
            // write it to the store state
            //
            logger.debug(`[EVA:SDK:LOGIC:COMPOSITE:${serviceName}] Call service ${serviceNameForGetShoppingCart}`, payload, serviceOptions)();
            try {
                const order = await endpoint.callService(Core.GetShoppingCart, {
                    AdditionalOrderDataOptions: settings.shoppingCartOptions,
                    IncludeAllPayments: false,
                    ...payload.data.order,
                }, serviceOptions);
                const responsePayload = {
                    serviceContext: payload.serviceContext,
                    chainContext: payload.chainContext,
                    data: {
                        order,
                        status: determineOrderStatus({
                            requiredData: order.AdditionalOrderData?.RequiredData || {},
                            productRequirements: order.AdditionalOrderData?.ProductRequirements ?? undefined,
                            order: order.ShoppingCart,
                        }),
                        productCount: countOrderProductsByType({ order: order.ShoppingCart }),
                    },
                };
                return responsePayload;
            }
            catch (error) {
                logger.error(`[EVA:SDK:LOGIC:COMPOSITE:${serviceName}] Service error`, error)();
                const errorPayload = {
                    serviceContext: payload.serviceContext,
                    chainContext: payload.chainContext,
                    data: error,
                };
                // If this an HTTPError from fetch we can try to extract the JSON response
                //
                try {
                    const httpError = error;
                    const jsonError = await httpError.response?.json();
                    errorPayload.data = jsonError || httpError.message;
                    errorPayload.httpResponse = httpError.response?.status;
                    const headers = {};
                    httpError.response?.headers.forEach((value, key) => (headers[key.toLowerCase()] = value));
                    errorPayload.headers = headers;
                }
                catch (jsonParseError) {
                    logger.warn(`[EVA:SDK:LOGIC:${serviceName}] Failed to parse JSON error response`, jsonParseError)();
                }
                servicePromise.reject(errorPayload);
                throw errorPayload;
            }
        },
    });
    return [
        requestLogic,
    ];
}
//# sourceMappingURL=get-shopping-cart-with-status.js.map