import { logger } from '@springtree/eva-sdk-core-logger';
import { Interceptor, } from '@springtree/eva-sdk-core-service';
import { isFunction } from 'lodash-es';
import { createCurrentUserTokenAction } from '../eva-sdk-redux-store-builder.js';
/**
 * This interceptor listens to service responses for a 401 status and checks
 * if the current user token needs to be unset because it has expired
 *
 * @export
 * @class UserExpiredInterceptor
 */
export class UserExpiredInterceptor extends Interceptor {
    store;
    userTokenExpired;
    /**
     * Creates an instance of the EvaAvailableServices interceptor
     */
    constructor(store, userTokenExpired) {
        super('EVA:ReduxStoreBuildUserExpired');
        this.store = store;
        this.userTokenExpired = userTokenExpired;
        this.priority = 5;
    }
    /**
     * The response interceptor implementation.
     * Available as a public property for direct use
     * By returning a fetch Response we will prevent the service call from going out
     *
     * @type {IResponseInterceptor}
     */
    responseInterceptor = {
        hook: async (request, options, response) => {
            if (response.status === 401) {
                const storeState = this.store.getState();
                const storeToken = storeState.currentUserToken;
                const token = request.headers.get('Authorization');
                const serviceName = request.headers.get('EVA-Service-Name');
                const isLogin = serviceName === 'Core:Login';
                // Check the token is for this store
                // We can have multiple stores active with their service interceptors
                // Exclude the login service explicitly
                //
                if (!isLogin && token && token === storeToken) {
                    logger.log(`[EVA:SDK] stored user token has expired: ${token}`)();
                    const expiredAction = {
                        type: 'USER_TOKEN_EXPIRED',
                        payload: { token },
                    };
                    this.store.dispatch(expiredAction);
                    const unsetTokenAction = createCurrentUserTokenAction();
                    this.store.dispatch(unsetTokenAction);
                    logger.warn(`User token ${token} has expired. Clearing current user state`)();
                    await unsetTokenAction.payload.promiseChain;
                    if (isFunction(this.userTokenExpired)) {
                        this.userTokenExpired(token);
                    }
                }
                else {
                    logger.log(`[EVA:SDK] non-stored user token has expired: ${token}`)();
                }
            }
            // IMPORTANT: Do not return a response here or any other interceptors will fail to fire
        },
    };
}
//# sourceMappingURL=user-expired-interceptor.js.map