import { logger } from '@springtree/eva-sdk-core-logger';
import { createLogic } from 'redux-logic';
import { createServiceRequestAction } from '../action/index.js';
/**
 * Creates a Redux logic to link 2 EVA service actions together
 *
 * @export
 * @template SHAPE The Redux store shape
 * @template SVC_FROM The EVA Service repository definition to link from
 * @template SVC_TO The EVA Service repository definition to link to
 * @param {IChainOptions<SVC_FROM, SVC_TO>} params The chain link configuration
 * @returns {Logic[]}
 */
export function createChainLogic(params) {
    // Determine the state key name either by the service name or the provided
    // custom state key name
    //
    const fromServiceName = ((params && params.from && params.from.stateOptions && params.from.stateOptions.stateKey) || new params.from.service().name);
    const toServiceName = ((params && params.to && params.to.stateOptions && params.to.stateOptions.stateKey) || new params.to.service().name);
    const chainLogic = createLogic({
        type: `${String(fromServiceName)}:${params.from.type}`,
        processOptions: {
            failType: `${String(toServiceName)}:ERROR`,
        },
        async process({ getState, action }, dispatch, done) {
            // Pull the from service state from the store
            //
            const storeState = getState();
            const fromServiceState = storeState[fromServiceName];
            // Check if the caller provided a validate check which may prevent the link from triggering
            //
            if (params.to && params.to.validate && !params.to.validate(fromServiceState)) {
                logger.debug('[EVA:SDK:LOGIC] Not calling due to validation failure', params)();
            }
            else {
                // For the to service there is only type request
                // A response type makes no sense since that could have been achieved with
                // a reducer
                //
                // Use the endpoint and call options from the original request
                //
                const endpointUrl = params.to?.endpoint?.endPointUri ?? fromServiceState.endpoint ?? storeState.endpointUrl;
                const serviceOptions = params.to?.serviceOptions ?? fromServiceState.serviceOptions;
                // Create the request payload using the optionally provided transform function
                // Most services will chain with minimal details being transferred
                //
                const requestPayload = params.to && params.to.transform ? params.to.transform(fromServiceState) : undefined;
                // Create the request action payload
                //
                const requestAction = createServiceRequestAction({
                    serviceOptions,
                    endpoint: endpointUrl || '',
                    service: params.to.service,
                    payload: requestPayload,
                    stateOptions: { stateKey: toServiceName },
                    chainContext: action.payload.chainContext,
                });
                logger.debug('[EVA:SDK:LOGIC] Linked request', requestAction)();
                dispatch(requestAction);
            }
            // Redux logic needs this and if you omit it you will get a timeout warning
            //
            done();
        },
    });
    return [
        chainLogic,
    ];
}
//# sourceMappingURL=chain.js.map