import { Core } from '@springtree/eva-services-core';
import { createLogic } from 'redux-logic';
import { createServiceRequestLogic } from './request.js';
import { createCurrentUserTokenAction, createServiceRequestAction } from '../action/index.js';
import { getServicePromise } from '../promise.js';
/**
 * Creates all the EVA current user logic
 *
 * @export
 * @template SVC
 * @param {new () => SVC} serviceDefinition The EVA Service registry definition
 * @param {string} [stateKey] Optional state key name if not the same as the service name
 * @returns {Logic<IEvaServiceState<SVC>, object, object, object, any, string>}
 */
export function createCurrentUserLogic() {
    // Create the service request logics
    // These will handle the effect of actually calling the EVA services
    //
    const loginLogic = createServiceRequestLogic(Core.Login, { stateKey: 'login' });
    const logoutLogic = createServiceRequestLogic(Core.Logout, { stateKey: 'logout' });
    const currentUserLogic = createServiceRequestLogic(Core.GetCurrentUser, { stateKey: 'currentUser' });
    const registerLogic = createServiceRequestLogic(Core.CreateCustomer, { stateKey: 'register' });
    // Setting the user token will trigger a current user fetch
    // Do no remove successType here. It somehow breaks the flow :(
    //
    const setTokenUpdateUserLogic = createLogic({
        type: 'CURRENT_USER_TOKEN',
        processOptions: {
            dispatchReturn: true,
            successType: 'currentUser:REQUEST',
        },
        async process({ action }) {
            // If a new token is set we need to trigger a fetch for the current user data
            // There is no payload and the current Authorization token header will be set
            // from the current state
            //
            const payload = action.payload;
            const currentUserAction = createServiceRequestAction({
                service: Core.GetCurrentUser,
                chainContext: payload.chainContext,
            });
            // Push to the end of the call stack to let the reducers finish
            // and any related chains start
            //
            const callPromise = getServicePromise(payload.serviceContext);
            if (callPromise) {
                setTimeout(() => callPromise.resolve(payload.token || ''), 0);
            }
            return currentUserAction.payload;
        },
    });
    // A successful login will trigger a set token action which in turn will
    // trigger the current user fetch
    //
    const loginUpdateUserLogic = createLogic({
        type: 'login:RESPONSE',
        processOptions: {
            dispatchReturn: true,
        },
        async process({ getState, action }) {
            // Check if the login response was a success
            //
            const storeState = getState();
            const loginState = storeState.login;
            if (loginState?.data?.response?.User?.AuthenticationToken &&
                loginState.data.response.Authentication === 2 // EVA.Core.AuthenticationResults.Authenticated
            ) {
                const setTokenAction = createCurrentUserTokenAction({
                    token: loginState.data.response.User.AuthenticationToken,
                    chainContext: action.payload.chainContext,
                });
                return setTokenAction;
            }
            // No dispatch needed
            //
            return;
        },
    });
    // A successful or failed logout will trigger a set token action which in turn will
    // trigger the current user fetch
    // To token will be set to undefined
    //
    const logoutUpdateUserLogic = createLogic({
        type: ['logout:RESPONSE', 'logout:ERROR'],
        processOptions: {
            dispatchReturn: true,
        },
        async process({ action }) {
            // Always dispatch
            //
            const unsetTokenAction = createCurrentUserTokenAction({
                chainContext: action.payload.chainContext,
            });
            return unsetTokenAction;
        },
    });
    // Registering a customer with a returned Authentication token will trigger
    // a current user fetch
    //
    const registerAutoLoginLogic = createLogic({
        type: 'register:RESPONSE',
        processOptions: {
            dispatchReturn: true,
        },
        async process({ getState, action }) {
            // Check if the register response contains a token
            //
            const storeState = getState();
            const registerState = storeState.register;
            if (registerState &&
                registerState.data &&
                registerState.data.response &&
                registerState.data.response.User &&
                registerState.data.response.User.AuthenticationToken) {
                const setTokenAction = createCurrentUserTokenAction({
                    token: registerState.data.response.User.AuthenticationToken,
                    chainContext: action.payload.chainContext,
                });
                return setTokenAction;
            }
            // No dispatch needed
            //
            return;
        },
    });
    return [
        ...loginLogic,
        ...logoutLogic,
        ...currentUserLogic,
        ...registerLogic,
        setTokenUpdateUserLogic,
        loginUpdateUserLogic,
        logoutUpdateUserLogic,
        registerAutoLoginLogic,
    ];
}
//# sourceMappingURL=current-user.js.map