import { logger } from '@springtree/eva-sdk-core-logger';
import { bootstrapEndpoint } from '@springtree/eva-sdk-core-service';
import { createLogic } from 'redux-logic';
import { getServicePromise } from '../promise.js';
/**
 * Factory method for generic EVA service logic
 *
 * @export
 * @template SVC
 * @param {new () => SVC} serviceDefinition The EVA Service registry definition
 * @param {string} [stateKey] Optional state key name if not the same as the service name
 * @returns {Logic<IEvaServiceState<SVC>, any, any, any, any, string>}
 */
export function createServiceRequestLogic(serviceDefinition, stateOptions) {
    // Determine the state key name either by the service name or the provided
    // custom state key name
    //
    const serviceName = (stateOptions && stateOptions.stateKey) || new serviceDefinition().name;
    const requestLogic = createLogic({
        type: `${serviceName}:REQUEST`,
        processOptions: {
            dispatchReturn: true,
            successType: `${serviceName}:RESPONSE`,
            failType: `${serviceName}:ERROR`,
        },
        async process({ getState, action }) {
            const storeState = getState();
            const payload = action.payload;
            const endpointUrl = payload.endpoint || storeState.endpointUrl;
            const servicePromise = getServicePromise(payload.serviceContext);
            // Ensure we have an endpoint to use
            //
            if (!endpointUrl) {
                const failure = new Error('No EVA endpoint provided for service call');
                servicePromise.reject(failure);
                throw failure;
            }
            // Bootstrapping the store to get the EvaEndpoint instance we need
            // The SDK caches bootstrap results so after the first call this will be instant
            //
            const endpoint = await bootstrapEndpoint({ uri: endpointUrl });
            // Set provided chain context header
            //
            const serviceOptions = Object.assign({
                serviceContext: payload.serviceContext,
                chainContext: payload.chainContext,
            }, payload.serviceOptions);
            // Set the current user authentication token if needed
            //
            const currentUserToken = storeState.currentUserToken;
            if (!serviceOptions.authenticationToken && currentUserToken) {
                serviceOptions.authenticationToken = currentUserToken;
            }
            // Set the current application token if needed
            //
            const currentApplicationToken = storeState.currentApplicationToken;
            if (!serviceOptions.applicationToken && currentApplicationToken) {
                serviceOptions.applicationToken = currentApplicationToken;
            }
            // Call the service and return the response so the reducers can
            // write it to the store state
            //
            const serviceName = new serviceDefinition().name;
            logger.debug('[EVA:SDK:LOGIC] Call service', serviceName, payload, serviceOptions)();
            // Not logged in current user calls will be resolve as empty
            //
            if (serviceName === 'Core:GetCurrentUser' && !currentUserToken) {
                const responsePayload = {
                    serviceContext: payload.serviceContext,
                    chainContext: payload.chainContext,
                    data: undefined,
                };
                return responsePayload;
            }
            try {
                const result = await endpoint.callService(serviceDefinition, payload.data, serviceOptions);
                const responsePayload = {
                    serviceContext: payload.serviceContext,
                    chainContext: payload.chainContext,
                    data: result,
                };
                return responsePayload;
            }
            catch (error) {
                logger.error('[EVA:SDK:LOGIC] Service error', error)();
                const errorPayload = {
                    serviceContext: payload.serviceContext,
                    chainContext: payload.chainContext,
                    data: error,
                };
                // If this an HTTPError from fetch we can try to extract the JSON response
                //
                try {
                    const httpError = error;
                    const jsonError = await httpError.response?.json();
                    errorPayload.data = jsonError || httpError.message;
                    errorPayload.httpResponse = httpError.response?.status;
                    const headers = {};
                    httpError.response?.headers.forEach((value, key) => (headers[key.toLowerCase()] = value));
                    errorPayload.headers = headers;
                }
                catch (jsonParseError) {
                    logger.warn(`[EVA:SDK:LOGIC:${serviceName}] Failed to parse JSON error response`, jsonParseError)();
                }
                logger.error('[EVA:SDK:LOGIC] Error payload', errorPayload)();
                servicePromise.reject(errorPayload);
                throw errorPayload;
            }
        },
    });
    return [
        requestLogic,
    ];
}
//# sourceMappingURL=request.js.map