import { Core } from '@springtree/eva-services-core';
import { createServiceReducer } from './service.js';
/**
 * Creates a reducer to update the current user token
 *
 * @export
 * @returns {Reducer<string, IEvaCurrentUserTokenAction>}
 */
export function createEvaCurrentUserTokenReducer() {
    const reducer = (state = '', action) => {
        switch (action.type) {
            case 'CURRENT_USER_TOKEN': {
                return action?.payload?.token ?? '';
            }
            default: {
                return state;
            }
        }
    };
    return reducer;
}
/**
 * Creates a reducer to update the current application token
 *
 * @export
 * @returns {Reducer<string, IEvaAppTokenAction>}
 */
export function createEvaApplicationTokenReducer() {
    const reducer = (state = '', action) => {
        switch (action.type) {
            case 'APPLICATION_TOKEN': {
                const payload = action.payload;
                return payload && payload.token || '';
            }
            default: {
                return state;
            }
        }
    };
    return reducer;
}
/**
 * Create the reducer to extract the user status from the GetCurrentUser response
 *
 * @export
 * @returns {Reducer<IEvaCurrentUserStatus, TEvaCurrentUserActions>}
 */
export function createEvaCurrentUserStatusReducer() {
    const reducer = (state = {}, action) => {
        switch (action.type) {
            case 'currentUser:RESPONSE': {
                const payload = action.payload;
                const currentUser = payload.data;
                if (!currentUser) {
                    const newState = {
                        isAnonymous: true,
                        isLoggedIn: false,
                        isCustomer: false,
                        isEmployee: false,
                    };
                    return newState;
                }
                const userType = currentUser.User.Type;
                const newState = {
                    isAnonymous: (userType & 4) !== 0,
                    isLoggedIn: ((userType & 2) !== 0) || ((userType & 1) !== 0),
                    isCustomer: (userType & 2) !== 0,
                    isEmployee: (userType & 1) !== 0,
                };
                return newState;
            }
            case 'currentUser:RESET': {
                const currentUser = action.state?.data.response;
                if (!currentUser) {
                    const newState = {
                        isAnonymous: true,
                        isLoggedIn: false,
                        isCustomer: false,
                        isEmployee: false,
                    };
                    return newState;
                }
                const userType = currentUser.User.Type;
                const newState = {
                    isAnonymous: (userType & 4) !== 0,
                    isLoggedIn: ((userType & 2) !== 0) || ((userType & 1) !== 0),
                    isCustomer: (userType & 2) !== 0,
                    isEmployee: (userType & 1) !== 0,
                };
                return newState;
            }
            default: {
                return state;
            }
        }
    };
    return reducer;
}
// Create and combine all the reducers involved with the current user functionality
//
export const currentUserReducers = {
    currentUserToken: createEvaCurrentUserTokenReducer(),
    currentUserStatus: createEvaCurrentUserStatusReducer(),
    currentUser: createServiceReducer(Core.GetCurrentUser, { stateKey: 'currentUser' }),
    login: createServiceReducer(Core.Login, { stateKey: 'login' }),
    logout: createServiceReducer(Core.Logout, { stateKey: 'logout' }),
    register: createServiceReducer(Core.CreateCustomer, { stateKey: 'register' }),
    currentApplicationToken: createEvaApplicationTokenReducer(),
};
//# sourceMappingURL=current-user.js.map