import { getServicePromise } from '../promise.js';
/**
 * Factory method for a generic EVA Service request and response reducer
 *
 * @export
 * @template SVC The service definition this state reducer is for
 * @param {new () => SVC} serviceDefinition The EVA Service registry definition
 * @param {ICreateServiceReducerOptions} [options] Creation options for the service reducer
 * @returns {(Reducer<IEvaServiceState<SVC>|undefined>)}
 */
export function createServiceReducer(serviceDefinition, options) {
    const reducer = (state = { data: new serviceDefinition() }, action) => {
        // Determine the state key name either by the service name or the provided
        // custom state key name
        //
        const serviceName = (options && options.stateKey) || new serviceDefinition().name;
        const preserveResponseOnError = !!options?.preserveResponseOnError;
        switch (action.type) {
            // Handle a service request:
            // * store the request
            // * unset previous error state
            // * set fetching state
            // * store used endpoint
            // * store used call options
            //
            case `${serviceName}:REQUEST`: {
                const payload = action.payload || {};
                const newState = {
                    data: new serviceDefinition(),
                };
                newState.data.request = payload.data;
                newState.data.response = state.data.response; // Preserve last response during request
                newState.endpoint = payload.endpoint;
                newState.serviceOptions = payload.serviceOptions;
                newState.fetching = true;
                newState.error = undefined;
                newState.serviceContext = payload.serviceContext;
                newState.chainContext = payload.chainContext;
                newState.chainService = payload.chainService;
                newState.requestedAt = +(new Date());
                return newState;
            }
            // Handle a service response:
            // * store the response if it matches latest request
            // * unset previous error state
            // * unset fetching state
            //
            case `${serviceName}:RESPONSE`: {
                const payload = action.payload;
                const newState = {
                    data: new serviceDefinition(),
                };
                newState.data.request = state.data.request;
                newState.data.response = payload.data;
                newState.error = undefined;
                newState.fetching = false;
                newState.serviceContext = state.serviceContext;
                newState.chainContext = state.chainContext;
                newState.chainService = state.chainService;
                newState.requestedAt = state.requestedAt;
                newState.responseAt = +(new Date());
                // Check if there is a registered promise for the service context
                // and resolve it
                // Push to the end of the call stack to let the reducers finish
                // and any related chains start
                //
                const callPromise = getServicePromise(payload.serviceContext);
                if (callPromise) {
                    setTimeout(() => callPromise.resolve(action.payload.data), 0);
                }
                return newState;
            }
            // Handle a service error:
            // * store the error response
            // * set error state
            // * unset fetching state
            //
            case `${serviceName}:ERROR`: {
                const payload = action.payload;
                const newState = {
                    data: new serviceDefinition(),
                };
                newState.data.request = state.data.request;
                newState.data.response = preserveResponseOnError ? state.data.response : undefined;
                newState.error = payload.data;
                newState.fetching = false;
                newState.serviceContext = state.serviceContext;
                newState.chainContext = state.chainContext;
                newState.chainService = state.chainService;
                newState.chainService = state.chainService;
                // Check if there is a registered promise for the context
                // and reject it
                //
                // Push to the end of the call stack to let the reducers finish
                // and any related chains start
                //
                const callPromise = getServicePromise(payload.serviceContext);
                if (callPromise) {
                    setTimeout(() => callPromise.reject(action.payload), 0);
                }
                return newState;
            }
            // Handle a RESET request:
            // * purge the state
            // * optionally set from supplied payload
            //
            case `${serviceName}:RESET`: {
                const payload = action.payload;
                if (payload && payload.state) {
                    // New state provided use that as the new state
                    //
                    return payload.state;
                }
                // Build a new service state
                //
                const newState = {
                    data: new serviceDefinition(),
                };
                return newState;
            }
            default: {
                return state;
            }
        }
    };
    return reducer;
}
//# sourceMappingURL=service.js.map