import { isEqual, pick } from 'lodash-es';
import { combineReducers, createStore, compose, applyMiddleware } from 'redux';
import { createLogicMiddleware } from 'redux-logic';
import { logger } from '@springtree/eva-sdk-core-logger';
import { currentUserReducers, endpointReducers, shoppingCartReducers } from './reducer/index.js';
import { createCurrentOrderLogic, createCurrentUserLogic, createShoppingCartLogic } from './logic/index.js';
import { createEndpointAction, createServiceResetAction, createCurrentUserTokenAction } from './action/index.js';
import { UserExpiredInterceptor } from './interceptor/user-expired-interceptor.js';
import { ApplicationTokenInterceptor } from './interceptor/application-token-interceptor.js';
import { currentOrderReducers } from './reducer/current-order.js';
/**
 * This is the class to help you configure and build an EVA focused Redux store
 *
 * @export
 * @class StoreBuilder
 */
export class StoreBuilder {
    /**
     * This interceptor will detect expired user tokens for the this store instance
     *
     * @private
     * @type {EvaReduxServiceInterceptor}
     */
    userExpiredInterceptor;
    /**
     * This interceptor will detect application tokens
     *
     * @private
     * @type {EvaReduxServiceInterceptor}
     */
    applicationTokenInterceptor;
    /**
     * These are the default properties persisted to storage
     * If callers want to override properties but keep the defaults they
     * can include these values
     *
     * @type {string[]}
     */
    defaultStorageProperties = [
        'endpointUrl',
        'currentUserToken',
        'currentApplicationToken',
    ];
    /**
     * Create a redux store based on the reducers added to the builder
     *
     * @param {*} [any] The initial state to use for the store
     * @returns
     */
    createStore(params, storageEventEmitter = window) {
        // Create the redux-logic middleware and enhancer
        // Look I really tried to get these any types out of here but the redux-logic package dictates them
        //
        const logicMiddleware = createLogicMiddleware([
            // eslint-disable-next-line @typescript-eslint/no-explicit-any
            ...createCurrentUserLogic(),
            // eslint-disable-next-line @typescript-eslint/no-explicit-any
            ...createShoppingCartLogic(),
            // eslint-disable-next-line @typescript-eslint/no-explicit-any
            ...createCurrentOrderLogic(),
            // eslint-disable-next-line @typescript-eslint/no-explicit-any
            ...(params.logics || []),
        ]);
        const middleware = applyMiddleware(logicMiddleware);
        // Compose the enhancers adding Redux developer tools support if available
        //
        const composeEnhancers = window?.__REDUX_DEVTOOLS_EXTENSION_COMPOSE__ || compose;
        const enhancer = composeEnhancers(middleware);
        // Add the service interceptor and EVA endpoint reducers to the provided map
        //
        const allReducers = Object.assign(params.reducers || {}, endpointReducers, currentUserReducers, shoppingCartReducers, currentOrderReducers);
        const rootReducer = combineReducers(allReducers);
        // If storage was provided we will apply the data stored in the prefixed keys
        // in the initialState object
        //
        const storageProvider = params.storage;
        const storageKeyPrefix = params.storageKeyPrefix || 'evaReduxCache';
        const storageProperties = params.storageProperties || this.defaultStorageProperties;
        // Preserved state keys look like `${storagePrefix}:${stateKey}`
        //
        const initialState = (params.initialState || {});
        if (storageProvider) {
            for (let i = 0, iLen = storageProvider.length; i < iLen; i += 1) {
                const key = storageProvider.key(i);
                if (key && key.indexOf(storageKeyPrefix) === 0) {
                    // Local storage key starts with our storage key prefix so process it
                    //
                    try {
                        const rawStoredState = storageProvider.getItem(key);
                        if (rawStoredState) {
                            const storedStoreState = JSON.parse(rawStoredState);
                            const stateProperty = key.split(':')[1];
                            if (storageProperties.indexOf(stateProperty) !== -1) {
                                initialState[stateProperty] = storedStoreState;
                            }
                        }
                    }
                    catch (storageError) {
                        logger.warn(`[EVA:SDK] Invalid store state in storage key ${key}`, storageError)();
                    }
                }
            }
            logger.debug('[EVA:SDK] storage -> initial state', initialState)();
        }
        // Create the Redux store now
        //
        const store = createStore(rootReducer, initialState, enhancer);
        // Hook up the service interceptor to capture user expirations (401)
        //
        this.userExpiredInterceptor = new UserExpiredInterceptor(store, params.userTokenExpired);
        this.userExpiredInterceptor.activate();
        // Hook up the service interceptor to capture application tokens (EVA-App-Token)
        //
        if (this.applicationTokenInterceptor) {
            this.applicationTokenInterceptor.deactivate();
            delete this.applicationTokenInterceptor;
        }
        this.applicationTokenInterceptor = new ApplicationTokenInterceptor(store);
        this.applicationTokenInterceptor.activate();
        // If a storage was provided we will setup a listeners on the created store
        // to save the requested set of properties
        //
        if (storageProvider) {
            // This will listen for store changes and save data to storage
            //
            // Redux store -> Storage
            //
            let previousState = JSON.parse(JSON.stringify(pick(store.getState(), storageProperties)));
            store.subscribe(() => {
                const newState = JSON.parse(JSON.stringify(pick(store.getState(), storageProperties)));
                for (const [stateProperty] of Object.entries(newState)) {
                    const newData = newState[stateProperty];
                    const oldData = previousState[stateProperty];
                    if (!isEqual(newData, oldData)) {
                        logger.debug('[EVA:SDK] Redux store -> storage', stateProperty, oldData, newData)();
                        // Data has changed so write the changes to storage
                        // Other browser windows for the same domain will receive a storage
                        // event for this change
                        //
                        storageProvider.setItem(`${storageKeyPrefix}:${stateProperty}`, JSON.stringify(newData));
                    }
                }
                previousState = newState;
            });
            // Subscribe to specific storage changes and update storage if needed
            // This will pull data from storage and add it to the Redux store
            //
            // Storage -> Redux store
            //
            try {
                storageEventEmitter.addEventListener('storage', (storageEvent) => {
                    // Check if the changed key starts with our prefix
                    //
                    if (storageEvent.key && storageEvent.key.indexOf(storageKeyPrefix) === 0) {
                        // This is an event for one of the Redux store properties
                        // We will use the RESET event to completely replace the state
                        // but only if the property is in our set of requested properties
                        //
                        const [, stateProperty] = storageEvent.key.split(':');
                        if (storageProperties.indexOf(stateProperty) !== -1) {
                            logger.debug('[EVA:SDK] Storage -> Redux store', storageEvent)();
                            try {
                                const newValue = JSON.parse(storageEvent.newValue);
                                // Emit event to replace current store state
                                // Storage event are fired per key and only if the value differs
                                //
                                switch (stateProperty) {
                                    case 'endpointUrl':
                                        {
                                            // The set endpoint action has no linked logic so we will not
                                            // need to use a reset action
                                            //
                                            const endpointAction = createEndpointAction({ endpoint: newValue });
                                            store.dispatch(endpointAction);
                                        }
                                        break;
                                    case 'currentUserToken':
                                        {
                                            // Setting the current user token will trigger a linked
                                            // GetCurrentUser and will reduce new user status
                                            // We will not pass this data around through storage to prevent
                                            // outdated user details persisting in various places
                                            //
                                            const userTokenAction = createCurrentUserTokenAction({
                                                token: newValue,
                                            });
                                            store.dispatch(userTokenAction);
                                        }
                                        break;
                                    case 'currentApplicationToken':
                                        {
                                            // Can be returned at any time from the backend
                                            //
                                            const applicationTokenAction = {
                                                type: 'APPLICATION_TOKEN',
                                                payload: { token: newValue },
                                            };
                                            store.dispatch(applicationTokenAction);
                                        }
                                        break;
                                    default:
                                        {
                                            // All `createServiceReducer` based keys support a RESET event
                                            // The default set of properties does not have one of these but
                                            // this could be useful in the future
                                            //
                                            const resetServiceAction = createServiceResetAction({
                                                stateKey: stateProperty,
                                                state: newValue,
                                            });
                                            store.dispatch(resetServiceAction);
                                        }
                                        break;
                                }
                            }
                            catch (storageError) {
                                logger.warn('[EVA:SDK] Invalid store state in storage', storageError)();
                            }
                        }
                    }
                });
            }
            catch (storageEventListenerError) {
                logger.warn('[EVA:SDK] Failed to register storage event listener', storageEventListenerError)();
            }
        }
        return store;
    }
}
//# sourceMappingURL=store.js.map